/**
 * @file 	DensityKernel.java
 * 
 * @date 	Jan 16, 2005
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.lmp.util.kernels;

import ca.mcgill.lmp.MeasurableSet;
import ca.mcgill.lmp.MeasurableFunction;
import ca.mcgill.lmp.Measure;
import ca.mcgill.lmp.State;
import ca.mcgill.lmp.TransitionKernel;
import ca.mcgill.lmp.approx.MonteCarlo;
import ca.mcgill.lmp.util.FunctionsFamily;

/**
 *
 */
public class DensityKernel implements TransitionKernel
{
    /**
     * Comment for <code>densityFunctions</code>
     */
    private FunctionsFamily densityFunctions;
    
    /**
     * Comment for <code>integrator</code>
     */
    private MonteCarlo integrator;
    
    /**
     * Comment for <code>mu</code>
     */
    private Measure mu;
    
    /**
     * @param densityFunctions
     * @param measure
     */
    public DensityKernel(FunctionsFamily densityFunctions, Measure measure)
    {
        this.densityFunctions = densityFunctions;
        integrator = new MonteCarlo(measure);
        mu = measure;
    }

    /**
     * @param s
     * @return
     */
    public Measure getTransitionMeasure(State source)
    {
        return new DensityMeasure(source);
    }

    /**
     * @param B
     * @return
     */
    public MeasurableFunction getTransitionFunction(MeasurableSet B)
    {
        return new DensityTransitionFunction(B);
    }
    
    public String toString()
    {
        return "Probability density function: " + densityFunctions.toString() + "\n"
        	+ "Measure used for integration: " + mu.toString();
        
    }
    
    /**
     * 
     *
     */
    class DensityMeasure implements Measure
    {
        /**
         * Comment for <code>densityFunction</code>
         */
        private MeasurableFunction densityFunction;
        
        /**
         * @param source
         */
        public DensityMeasure(State source)
        {
            this.densityFunction = densityFunctions.getFunction(source);
        }

        /**
         * @param B
         * @return
         */
        public double measure(MeasurableSet destination)
        {
            return integrator.integrate(densityFunction, destination);
        }

        /**
         * @param B
         * @return
         */
        public State sampleIn(MeasurableSet B)
        {
            throw new RuntimeException("Unimplemented method: DensityMeasure.sampleIn");
        }

        /**
         * @return
         */
        public MeasurableSet getX()
        {
            return mu.getX();
        } 
        
        /**
         * @return
         */
        public State sample()
        {
            throw new RuntimeException("ApproximationMeasure.sampleIn: not implemented.");
        } 
    }
    
    /**
     * 
     *
     */
    class DensityTransitionFunction implements MeasurableFunction
    {
        /**
         * Comment for <code>destination</code>
         */
        private MeasurableSet destination;
        
        /**
         * @param destination
         */
        public DensityTransitionFunction(MeasurableSet destination)
        {
            this.destination = destination;
        }

        /**
         * @param s
         * @return
         */
        public double evaluateAt(State source)
        {
            return integrator.integrate(densityFunctions.getFunction(source), destination);
        }
        
        public String toString()
        {
            return densityFunctions.toString() + "(., " + destination.toString() + ")";
        }
    }
}
